/*********************************************************************
*                    SEGGER Microcontroller GmbH                     *
*                        The Embedded Experts                        *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------

File    : main.c
Purpose : Blinky application for the Cortex-M Trace Reference Board
          Device: STM32F407VE
          LED0 connected to GPIOA_08

*/

/**********************************************************************************
*
*       Defines
*
**********************************************************************
*/

#define DELAY_CYCLES        (500000)

#define GPIOA_BASE_ADDR     ((unsigned int)0x40020000)
#define RCC_BASE_ADDR       ((unsigned int)(0x40023800))

#define RCC_AHB1RSTR        (*(volatile unsigned int*)(RCC_BASE_ADDR + 0x10))
#define RCC_AHB1ENR         (*(volatile unsigned int*)(RCC_BASE_ADDR + 0x30))

#define GPIOA_MODER         (*(volatile unsigned int*)(GPIOA_BASE_ADDR + 0x00))
#define GPIOA_BSRR          (*(volatile unsigned int*)(GPIOA_BASE_ADDR + 0x18))

#define LEDPORT_RCC_BIT     (0)
#define LED0_BIT            (8)

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/

/*********************************************************************
*
*       Static functions
*
**********************************************************************
*/

/*********************************************************************
*
*     _Delay
*/
static void _Delay(void) {
  int i;

  for (i = 0; i < DELAY_CYCLES; ++i) {
    ;
  }
}

/*********************************************************************
*
*       main()
*
*  Function description
*   Application entry point.
*/
int main(void) {
  int i;
  //
  // Initialize port for LED0 (sample application)
  //
  RCC_AHB1ENR  &= ~(1uL << LEDPORT_RCC_BIT);
  RCC_AHB1RSTR &= ~(1uL << LEDPORT_RCC_BIT);
  RCC_AHB1ENR  |=  (1uL << LEDPORT_RCC_BIT);
  //
  // Initialze GPIO pin of LED0
  //
  GPIOA_BSRR   = (1uL << LED0_BIT);         // Initially clear LED
  GPIOA_MODER &= ~(3uL << (LED0_BIT * 2));  // Reset mode; sets port to input
  GPIOA_MODER |=  (1uL << (LED0_BIT * 2));  // Set to output mode
  //
  // LED toggle loop
  //
  do {
    GPIOA_BSRR = ((1uL << LED0_BIT) << 16);  // Set LED
    _Delay();
    GPIOA_BSRR = (1uL << LED0_BIT);          // Clear LED
    _Delay();
    i++;
  } while (1);
}

/*************************** End of file ****************************/
